import django_tables2 as tables
from dbsynce.models import *
from dbsynce.models import *
from django.contrib.auth import get_user_model
from django.utils.html import format_html
from django.utils.translation import gettext_lazy as _


class TransactionsWalletTable(tables.Table):
    # id = tables.Column(order_by=True)
    id = tables.Column(
        verbose_name='#',
        orderable=False,
        attrs={
            "td": {"width": "5%"}
        }
    )
    wallet = tables.Column(
        verbose_name=_('Owner'),
        orderable=False,
        attrs={
            "td": {"width": "15%"}
        }
    )
    name_operation = tables.Column(
        verbose_name=_('Service'),
        attrs={
            'th': {'scope': 'col'},
            "td": {"width": "20%"}
        }
    )
    price = tables.Column(
        verbose_name=_('Points'),
        attrs={
            "class": "row",
            "td": {"width": "10%"}
        }
    )
    date_operation = tables.Column(
        verbose_name=_('Registration date'),
        attrs={
            "td": {"width": "30%"}
        }
    )
    is_carried_out = tables.BooleanColumn(
        verbose_name=_('Status'),
        orderable=False,
        yesno=_("Successful, not successful"),
        attrs={
            "td": {"width": "20%"}
        }
    )

    class Meta:
        # model = TransactionsWallets
        attrs = {
            "class": "table table-striped"
        }
        exclude = (
            "balance_before",
            "amount",
            "metaservice_id",
            "transaction_type",
            "doc_num",
            "service_id"
        )

    def render_name_operation(self, value, record):
        return format_html(
            "<a href='{}'>{}</a>",
            record.get_absolute_url(),
            value
        )


class PartnersTable(tables.Table):
    id = tables.Column(
        verbose_name=_('ID'),
        attrs={
            "td": {"width": "5%"}
        }
    )
    legal_name = tables.Column(
        verbose_name=_('Legal entity'),
        attrs={
            'th': {'scope': 'col'},
            "td": {"width": "20%"}
        }
    )
    repr = tables.Column(
        accessor='repr.full_name',
        order_by=('repr.first_name', 'repr.last_name'),
        verbose_name=_('Responsible'),
        attrs={
            "td": {"width": "15%"}
        }
    )
    status = tables.Column(
        verbose_name=_('Status'),
        attrs={
            'th': {'scope': 'col'},
            "td": {"width": "20%"}
        }
    )
    check = tables.BooleanColumn(
        verbose_name='',
        attrs={
            'th': {'scope': 'col'},
            "td": {"width": "20%"}
        }
    )

    # paginate_by = 10
    class Meta:
        model = Company
        attrs = {
            "class": "table table-layout-fixed"
        }
        exclude = (
            'inn',
            'kpp',
            'ogrn',
            'bank_name',
            'bik',
            'ks',
            'rs',
            'address',
            'requirements',
            'id_metaservice',
            'is_global',
            'is_visible',
            'ticket_status'
        )

    def render_check(self, value, record):
        if record.status == 'active':
            return format_html(
                '<input class="form-check-input status-toggle" checked type="checkbox" id="flexCheckDefault" data-partners-id="{}">',
                record.id
            )
        else:
            return format_html(
                '<input class="form-check-input status-toggle" type="checkbox" id="flexCheckDefault" data-partners-id="{}">',
                record.id
            )

class OrdersTable(tables.Table):
    id = tables.Column(
        verbose_name=_('ID'),
        attrs={
                "td": {"width": "5%"}
        }
    )
    title = tables.Column(
        verbose_name=_('Title'), 
        attrs={
            "td": {"width": "20%"}
        }
    )
    status = tables.Column(
        verbose_name=_('Status'), 
        attrs={
            "td": {"width": "15%"}
        }
    )
    receiver = tables.Column(
        accessor='receiver.get_full_name',
        verbose_name=_('Receiver'),
        attrs={
            "td": {"width": "20%"}
        }
    )
    time_created = tables.DateTimeColumn(
        verbose_name=_('Created'),
        attrs={
            "td": {"width": "20%"}
        }
    )

    class Meta:
        model = Orders
        attrs = {
            "class": "table table-layout-fixed"
        }
        exclude = (
            'id_metaservice', 
            'note', 
            'time_placed', 
            'time_start', 
            'time_start_real', 
            'time_start_predicted', 
            'time_finish_real', 
            'time_finish_predicted', 
            'real_price', 
            'predicted_price', 
            'asap', 
            'is_global', 
            'is_visible', 
            'ticket', 
            'service', 
            'service_category', 
            'client', 
            'provider', 
            'company')

class ResourcesTable(tables.Table):
    id = tables.Column(
        verbose_name=_('ID'),
        attrs={
            "td": {"width": "5%"}
        }
    )
    # В user ссылка LinkColumn на страницу Аси "Информация о партнере" страница partner_information_form
    user = tables.Column(
        accessor='user.get_full_name',
        order_by=('user.first_name', 'user.last_name'),
        verbose_name=_('Responsible'),
        attrs={
            "td": {"width": "15%"}
        }
    )
    status = tables.Column(
        verbose_name=_('Status'),
        attrs={
            'th': {'scope': 'col'},
            "td": {"width": "20%"}
        }
    )
    check = tables.BooleanColumn(
        verbose_name='',
        attrs={
            'th': {'scope': 'col'},
            "td": {"width": "40%"}
        }
    )

    # paginate_by = 10
    class Meta:
        model = Resource
        attrs = {
            "class": "table table-layout-fixed"
        }
        exclude = (
            'id_metaservice',
            'resource_type',
            'requirements',
            'is_global',
            'is_visible',
            'ticket_status'
        )

    def render_check(self, value, record):
        if record.status == 'active':
            return format_html(
                '<input class="form-check-input status-toggle" checked type="checkbox" id="flexCheckDefault" data-resource-id="{}">',
                record.id
            )
        else:
            return format_html(
                '<input class="form-check-input status-toggle" type="checkbox" id="flexCheckDefault" data-resource-id="{}">',
                record.id
            )

class ProvidersTable(tables.Table):
    id = tables.Column(
        verbose_name=_('ID'),
        attrs={
            "td": {"width": "5%"}
        }
    )
    user = tables.Column(
        accessor='user.full_name',
        order_by=('user.first_name', 'user.last_name'),
        verbose_name=_('Full Name'),
        attrs={
            "td": {"width": "15%"}
        }
    )
    status = tables.Column(
        verbose_name=_('Status'),
        attrs={
            'th': {'scope': 'col'},
            "td": {"width": "20%"}
        }
    )
    check = tables.BooleanColumn(
        verbose_name='',
        attrs={
            'th': {'scope': 'col'},
            "td": {"width": "20%"}
        }
    )
    paginate_by = 10

    class Meta:
        model = Provider
        attrs = {
            "class": "table table-layout-fixed"
        }
        exclude = (
            'id_metaservice',
            'gap',
            'requirements',
            'status',
            'service_status',
            'location_type',
            'default_location',
            'is_global',
            'is_visible'
        )

    def render_check(self, value, record):
        if record.status == 'active':
            return format_html(
                '<input class="form-check-input status-toggle" checked type="checkbox" id="flexCheckDefault" data-provider-id="{}">',
                record.id
            )
        else:
            return format_html(
                '<input class="form-check-input status-toggle" type="checkbox" id="flexCheckDefault" data-provider-id="{}">',
                record.id
            )


class ServiceTariffsTable(tables.Table):
    id = tables.Column(
        verbose_name=_('ID'),
        attrs={
            "td": {"width": "5%"}
        }
    )
    codename = tables.LinkColumn(
        'sharix_admin:service_tariff/edit/',
        verbose_name=_('Кодовое название'),
        text=lambda record: record.caption,
        args=[tables.A('pk')],
        attrs={
            'th': {'scope': 'col'},
            "td": {"width": "15%"}
        }
    )
    #service_category = tables.Column(
    service_category = tables.LinkColumn(
        'sharix_admin:service_category/edit/',
        verbose_name=_('Название схемы услуги'),
        text=lambda record: record.service_category.caption,
        args=[tables.A('pk')],
        attrs={
            'th': {'scope': 'col'},
            "td": {"width": "15%"}
        }
    )
    resource_type = tables.Column(
        verbose_name=_('Тип ресурса'),
        attrs={
            'th': {'scope': 'col'},
            "td": {"width": "20%"}
        }
    )

    company_comission = tables.Column(
        verbose_name=_('Комиссия партнера'),
        attrs={
            'th': {'scope': 'col'},
            "td": {"width": "20%"}
        }
    )

    check = tables.BooleanColumn(
        verbose_name=_('Activity'),
        orderable=False,
        attrs={
            'th': {'scope': 'col'},
            "td": {"width": "20%"}
        }
    )

    class Meta:
        model = Service
        attrs = {
            "class": "table table-layout-fixed"
        }
        exclude = (
            'resource',
            'company',
            'caption',
            'description',
            'price_type',
            'price_min',
            'price_amount',
            'id_metaservice',
            'requirements',
            'ticket_status',
            'price_km',
            'is_global',
            'is_visible',
            'status'
        )

    def render_check(self, value, record):
        if record.status == 'active':
            return format_html('<input class="form-check-input status-toggle" disabled  checked type="checkbox"')
        else:
            return format_html('<input class="form-check-input status-toggle" disabled  type="checkbox"')


class ServiceCategoriesTable(tables.Table):
    id = tables.Column(attrs={"td": {"width": "50px"}})

    caption = tables.LinkColumn(
        'sharix_admin:service_category/edit/',
        verbose_name='Услуга',
        orderable=False,
        text=lambda record: record.caption,
        args=[tables.A('pk')],
        attrs={
            "a": {"style": "pointer-events: none;"},
            'th': {'scope': 'col'},
            "td": {"class": "name_col"}
        }
    )

    #codename = tables.LinkColumn(
    #    'sharix_admin:service_category/edit/',
    #    verbose_name='Тех.название',
    #    orderable=False,
    #    text=lambda record: record.codename,
    #    args=[tables.A('pk')],
    #    attrs={
    #        "a": {"style": "pointer-events: none;"},
    #        'th': {'scope': 'col'},
    #        "td": {"class": "name_col"}
    #    }
    #)
    description = tables.LinkColumn(
        'sharix_admin:service_category/edit/',
        orderable=False, verbose_name='Описание',
        text=lambda record: record.description,
        args=[tables.A('pk')],
        attrs={
            "a": {"style": "pointer-events: none;"},
            'th': {'scope': 'col'},
            "td": {"class": "description_col"}
        }
    )

    metaservice_comission = tables.Column(
        verbose_name=_('Комиссия Сервиса'),
        attrs={
            'th': {'scope': 'col'},
            "td": {"width": "15%"}}
    )

    edit = tables.LinkColumn(
        'sharix_admin:service_category/edit/',
        verbose_name='',
        orderable=False,
        text="E",
        args=[tables.A('pk')],
        attrs={
            'th': {'scope': 'col'},
            "td": {"class": "edit_col"}
        }
    )
    
    check = tables.BooleanColumn(
        verbose_name=_('Activity'),
        orderable=False,
        attrs={
            'th': {'scope': 'col'},
            "td": {"width": "5%"}
        }
    )
    #TODO change to 'change status!'
    #deletee = tables.LinkColumn(
    #    'sharix_admin:service_category/delete/',
    #    verbose_name='',
    #    orderable=False,
    #    text="D",
    #    args=[tables.A('pk')],
    #    attrs={
    #        'th': {'scope': 'col'},
    #        "td": {"class": "delete_col"}
    #    }
    #)

    class Meta:
        model = ServiceCategory
        attrs = {
            "class": "table table-layout-fixed text-start"
        }
        exclude = (
            'requirements',
            'price_type',
            'status',
            'ticket_status',
            'id_metaservice',
            'link_agreement',
            'is_global',
            'is_visible',
            'codename',
        )

    def render_check(self, value, record):
        if record.status == '0':
            #if record.status == 'active':
            return format_html(
                '<input class="form-check-input status-toggle" checked type="checkbox" id="flexCheckDefault" data-service_category-id="{}">',
                record.id
            )
            #return format_html('<input class="form-check-input status-toggle" checked type="checkbox" id="flexCheckDefault" data-service_category-id="{}">')
            #return format_html('<input class="form-check-input status-toggle" disabled  checked type="checkbox"')
        else:
            return format_html(
                '<input class="form-check-input status-toggle" type="checkbox" id="flexCheckDefault" data-service_category-id="{}">',
                record.id
            )
            #return format_html('<input class="form-check-input status-toggle" disabled  type="checkbox"')

    # def render_delete(self, value, record):
    #     return format_html('<a href="/service_category/delete" class="btn btn-outline-danger">_(Delete)</a>')

    # def render_name_operation(self, value, record):
    #     return format_html("<a href='{}'>{}</a>", record.get_absolute_url(), value)


class ServiceTable(tables.Table):
    id = tables.Column(
        verbose_name=_('ID'),
        attrs={
            "td": {"width": "5%"}
        }
    )
    service_category = tables.Column(
        verbose_name=_('Description of the service'),
        accessor='service_category.caption',
        attrs={
            'th': {'scope': 'col'},
            "td": {"width": "20%"}}
    )

    # description = tables.Column(verbose_name='Название тарифа', attrs={'th':{'scope':'col'}, "td":{"width":"20%"}})
    # description = tables.Column(verbose_name='Описание строки тарифов', attrs={'th':{'scope':'col'}, "td":{"width":"20%"}})
    # price_type = tables.Column(verbose_name='Тип тарифа', attrs={'th':{'scope':'col'}, "td":{"width":"20%"}})

    price_km = tables.Column(
        verbose_name=_('Cost km.'),
        attrs={
            'th': {'scope': 'col'},
            "td": {"width": "20%"}
        }
    )
    price_min = tables.Column(
        verbose_name=_('Cost min.'),
        attrs={
            'th': {'scope': 'col'},
            "td": {"width": "20%"}}
    )
    price_amount = tables.Column(
        verbose_name=_('Cost of service'),
        attrs={
            'th': {'scope': 'col'},
            "td": {"width": "20%"}
        }
    )

    class Meta:
        model = Service
        attrs = {
            "class": "table table-layout-fixed"
        }
        exclude = (
            'resource',
            'requirements',
            'id_metaservice',
            'price_type',
            'ticket_status',
            'is_global',
            'is_visible'
        )

    def render_check(self, value, record):
        if record.status == 'active':
            return format_html(
                '<input class="form-check-input status-toggle" checked type="checkbox" id="flexCheckDefault" data-service-id="{}">',
                record.id
            )
        else:
            return format_html(
                '<input class="form-check-input status-toggle" type="checkbox" id="flexCheckDefault" data-service-id="{}">',
                record.id
            )


class UserInfoTable(tables.Table):
    id = tables.Column(
        verbose_name=_('ID'),
        attrs={
            "td": {"width": "5%"}
        }
    )

    class Meta:
        model = get_user_model()
        attrs = {
            "class": "table table-layout-fixed"
        }
        exclude = (
            'password',
            'phone_number',
            'last_login',
            'is_staff',
            'is_superuser',
            'date_joined'
        )


class PermissionsTable(tables.Table):
    id = tables.Column(
        verbose_name=_('ID'),
        attrs={
            "td": {"width": "5%"}
        }
    )
    user = tables.Column(
        accessor='user.full_name',
        order_by=('user.first_name', 'user.last_name'),
        verbose_name=_('User'),
        attrs={
            "td": {"width": "15%"}
        }
    )

    id_permissions = tables.Column(
        verbose_name=_('Permission'),
        attrs={
            "td": {"width": "15%"}
        }
    )

    status = tables.Column(
        verbose_name=_('Status'),
        attrs={
            'th': {'scope': 'col'},
            "td": {"width": "20%"}
        }
    )
    check = tables.BooleanColumn(
        verbose_name='',
        attrs={
            'th': {'scope': 'col'},
            "td": {"width": "20%"}
        }
    )

    # paginate_by = 10
    class Meta:
        model = Permissions
        attrs = {
            "class": "table table-layout-fixed"
        }
        exclude = (
            'check_date',
            'expire_date',
            'check_level',
            'checked_by',
            'ticket_status'
        )

    def render_check(self, value, record):
        if record.status == 'active':
            return format_html(
                '<input class="form-check-input status-toggle" checked type="checkbox" id="flexCheckDefault" data-resource-id="{}">',
                record.id
            )
        else:
            return format_html(
                '<input class="form-check-input status-toggle" type="checkbox" id="flexCheckDefault" data-resource-id="{}">',
                record.id
            )


class RelationshipTable(tables.Table):
    id = tables.Column(
        verbose_name=_('ID'),
        attrs={
            "td": {"width": "5%"}
        }
    )
    user_who = tables.Column(
        accessor='user_who.full_name',
        order_by=('user_who.first_name', 'user_who.last_name'),
        verbose_name=_('Initiator'),
        attrs={
            "td": {"width": "15%"}
        }
    )
    user_whom = tables.Column(
        accessor='user_whom.full_name',
        order_by=('user_whom.first_name', 'user_whom.last_name'),
        verbose_name=_('Goal'),
        attrs={
            "td": {"width": "15%"}
        }
    )
    neg_type = tables.Column(
        verbose_name=_('Neg Type'),
        attrs={
            'th': {'scope': 'col'},
            "td": {"width": "20%"}
        }
    )
    status = tables.Column(
        verbose_name=_('Status'),
        attrs={
            'th': {'scope': 'col'},
            "td": {"width": "20%"}
        }
    )
    check = tables.BooleanColumn(
        verbose_name='',
        attrs={
            'th': {'scope': 'col'},
            "td": {"width": "20%"}
        }
    )
    paginate_by = 10

    class Meta:
        model = Relationship
        attrs = {
            "class": "table table-layout-fixed"
        }
        exclude = (
            'ticket_status',
        )

    def render_check(self, value, record):
        if record.status == 'active':
            return format_html(
                '<input class="form-check-input status-toggle" checked type="checkbox" id="flexCheckDefault" data-provider-id="{}">',
                record.id
            )
        else:
            return format_html(
                '<input class="form-check-input status-toggle" type="checkbox" id="flexCheckDefault" data-provider-id="{}">',
                record.id
            )
