from dbsynce.models import Company
from dbsynce.models import Relationship
from dbsynce.models import Permissions


from django.contrib.auth.decorators import login_required
from django.http import JsonResponse
from django.utils.translation import gettext as _
from django_tables2 import SingleTableView
from django.views.generic import TemplateView, DetailView

from sharix_admin.tables import PartnersTable
from sharix_admin.tables import PermissionsTable
from sharix_admin.tables import RelationshipTable
from sharix_admin.utils import group_required
from .base import BaseView

from django.contrib.auth import get_user_model
from django.contrib.auth.mixins import UserPassesTestMixin
from django.contrib.auth.models import Group
from django.utils.translation import gettext as _



#TODO This is abslutely incorrect, custom page needed!
class AccessControlListView(BaseView, UserPassesTestMixin, SingleTableView):
    table_class = PermissionsTable
    queryset = Permissions.objects.all()
    template_name = 'sharix_admin/staff/access_control.html'
    page_title = 'Управление правами доступа'
    page_name = 'access_control'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context.update({
            'title': _('User roles Management'),
            'object_list': context['object_list'],
            'groups': Group.objects.all()
        })
        return context

    def get_queryset(self):
        queryset = super().get_queryset()
        search_query = self.request.GET.get('q', '').strip()
        if search_query:
            queryset = queryset.filter(
                Q(id__icontains=search_query) |
                Q(user__icontains=search_query) |
                Q(checked_by__icontains=search_query) |
                Q(expire_date__icontains=search_query)
            ).distinct()
        return queryset

    def test_func(self) -> bool or None:
        group_names = ('METASERVICE-ADMIN')
        if bool(self.request.user.groups.filter(name=group_names)) or self.request.user.is_superuser:
            return True
        return False
    #    return True

class RelationshipListView(BaseView, SingleTableView):
    page_title = _('Relationship')
    page_name = 'relationship'
    table_class = RelationshipTable
    queryset = Relationship.objects.all()
    template_name = 'sharix_admin/staff/access_control.html'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context.update({
            'object_list': context['object_list'],
        })
        return context

    def get_queryset(self):
        queryset = super().get_queryset()
        search_query = self.request.GET.get('q', '').strip()
        if search_query:
            queryset = queryset.filter(
                Q(id__icontains=search_query) |
                Q(user_who__icontains=search_query) |
                Q(user_whom__icontains=search_query) |
                Q(neg_type__icontains=search_query)
            ).distinct()
        return queryset

    def test_func(self):
        group_names = ('METASERVICE-ADMIN')
        if bool(self.request.user.groups.filter(name=group_names)) or self.request.user.is_superuser:
            return True
        return False


@login_required
@group_required('PLATFORM-ADMIN')
def change_relationship_status(request):
    if request.method == 'POST':
        relationship_id = request.POST.get('relationship')
        new_status = request.POST.get('new_status')
        relationship = Relationship.objects.get(pk=relationship)
        relationship.status = new_status
        relationship.save()
        return JsonResponse({'status': 'success'})
    else:
        return JsonResponse({'status': 'error'})


@login_required
@group_required('PLATFORM-ADMIN')
def change_permissions_status(request):
    if request.method == 'POST':
        permissions_id = request.POST.get('permissions')
        new_status = request.POST.get('new_status')
        permissions = Permissions.objects.get(pk=permissions)
        permissions.status = new_status
        permissions.save()
        return JsonResponse({'status': 'success'})
    else:
        return JsonResponse({'status': 'error'})

